/**************************************************************************
Jennifer La'O and Alireza Tahbaz-Salehi
"Optimal Monetary Policy in Production Networks" 
Econometrica
November 2021
**************************************************************************

This file generates a workable dataset from raw data in 3 steps:
Step 1. Reads Make and Use Tables from BEA and drops government industries
Step 2. Merges the resulting dataset with BEA-KLEMS productivity data
Step 3. Merges the resulting dataset with price change frequency from Pasten et al. (2020)

** Notes **
1. The working directory has to be set to the main folder that contains this file
2. The raw data is read from subfolder "Raw_Data"
3. The merged datasets are saved in subfolder "Merged_Data"


All output files sort the industries in the following order:
1 .Farms
2. Forestry, fishing, and related activities
3. Oil and gas extraction
4. Mining, except oil and gas
5. Support activities for mining
6. Utilities
7. Construction
8. Wood products
9. Nonmetallic mineral products
10. Primary metals
11. Fabricated metal products
12. Machinery
13. Computer and electronic products
14. Electrical equipment, appliances, and components
15. Motor vehicles, bodies and trailers, and parts
16. Other transportation equipment
17. Furniture and related products
18. Miscellaneous manufacturing
19. Food and beverage and tobacco products
20. Textile mills and textile product mills
21. Apparel and leather and allied products
22. Paper products
23. Printing and related support activities
24. Petroleum and coal products
25. Chemical products
26. Plastics and rubber products
27. Wholesale trade
28. Motor vehicle and parts dealers
29. Food and beverage stores
30. General merchandise stores
31. Other retail
32. Air transportation
33. Rail transportation
34. Water transportation
35. Truck transportation
36. Transit and ground passenger transportation
37. Pipeline transportation
38. Other transportation and support activities
39. Warehousing and storage
40. Publishing industries, except internet (includes software)
41. Motion picture and sound recording industries
42. Broadcasting and telecommunications
43. Data processing, internet publishing, and other information services
44. Federal Reserve banks, credit intermediation, and related activities
45. Securities, commodity contracts, and investments
46. Insurance carriers and related activities
47. Funds, trusts, and other financial vehicles
48. Housing
49. Other real estate
50. Rental and leasing services and lessors of intangible assets
51. Legal services
52. Computer systems design and related services
53. Miscellaneous professional, scientific, and technical services
54. Management of companies and enterprises
55. Administrative and support services
56. Waste management and remediation services
57. Educational services
58. Ambulatory health care services
59. Hospitals
60. Nursing and residential care facilities
61. Social assistance
62. Performing arts, spectator sports, museums, and related activities
63. Amusements, gambling, and recreation industries
64. Accommodation
65. Food services and drinking places
66. Other services, except government
***************************************************************************/

/**************************************************************************
1. Load raw I-O tables from BEA and drop government industries
***************************************************************************/

** Load Make Table from BEA Raw Data

clear all
cd Raw_Data/BEA_IO_Data
use Make_Table_2019.dta


** Drop Government Industries and Scrap, and clean up the file

drop Federalgeneralgovernmentdefe
drop Federalgeneralgovernmentnond
drop Federalgovernmententerprises
drop Stateandlocalgeneralgovernme
drop Stateandlocalgovernmententer
drop Scrapusedandsecondhandgoods
drop Noncomparableimportsandresto

drop if IOCode == "GFGD"
drop if IOCode == "GFGN"
drop if IOCode == "GFE"
drop if IOCode == "GSLG"
drop if IOCode == "GSLE"
drop if IOCode == ""
drop if Name == ""


/* Dropped industries
GFGD: Federal general government (defense)	
GFGN: Federal general government (nondefense)
GFE: Federal government enterprises	
GSLG: State and local general government	
GSLE: State and local government enterprises	
Used: Scrap, used, and secondhand goods
*/

preserve
keep TotalIndustryOutput
cd ../../Merged_Data
save TotalIndustryOutput.dta, replace
export excel TotalIndustryOutput.xlsx, replace
restore


drop TotalIndustryOutput
outsheet IOCode Name using "IndustryList.csv" , comma replace noname 

drop Name
drop IOCode

save Make_Table_clean.dta, replace
export excel MakeTable.xlsx, replace

** Load Use Table from BEA Raw Data

clear all
cd ..
cd Raw_Data
cd BEA_IO_Data
use Use_Table_2019.dta

** Drop Government Industries and Scrap

drop Federalgeneralgovernmentdefe
drop Federalgeneralgovernmentnond
drop Federalgovernmententerprises
drop Stateandlocalgeneralgovernme
drop Stateandlocalgovernmententer

drop SumofIntermediateSelected IntermediateNotSelected TotalIntermediate Personalconsumptionexpenditure Nonresidentialprivatefixedinv CA CB Residentialprivatefixedinvest Changeinprivateinventories Exportsofgoodsandservices FederalnationaldefenseConsum CI CJ NondefenseConsumptionexpendit FederalnationalnondefenseGro CN CM StateandlocalConsumptionexp StateandlocalGrossinvestmen CQ CR SumofFinalUsesGDPSelected SumofFinalUsesGDPNotSele TotalCommodityOutput FederalnationaldefenseGross Importsofgoodsandservices

drop if IOCode == "GFGD"
drop if IOCode == "GFGN"
drop if IOCode == "GFE"
drop if IOCode == "GSLG"
drop if IOCode == "GSLE"
drop if IOCode == "Used"
drop if IOCode == "Other"

drop if Name == ""
drop if Name == "Sum of Intermediate Selected"
drop if Name == "Intermediate Not Selected"
drop if Name == "Total Intermediate"
drop if Name == "Taxes on production and imports, less subsidies"
drop if Name == "Gross operating surplus"
drop if Name == "Sum of Value Added Selected"
drop if Name == "Sum of Value Added Not Selected"
drop if Name == "Total Industry Output"

/* 
Save "Total Final Use"
*/

preserve
drop if Name == "Compensation of employees"
drop if Name == "Total Value Added"
keep TotalFinalUsesGDP
cd ../../Merged_Data
save FinalUse.dta, replace
export excel FinalUse.xlsx, replace
restore

/* 
Save the rows for "Total Value Added" and "Compensation of Employees". 
*/

drop TotalFinalUsesGDP
preserve
keep if Name == "Compensation of employees"
drop Name
drop IOCode
save CompensationEmployees.dta, replace
export excel CompensationEmployees.xlsx, replace
restore

drop if Name == "Compensation of employees"

preserve
keep if Name == "Total Value Added"
drop Name
drop IOCode 
save ValueAdded.dta, replace
export excel ValueAdded.xlsx, replace
restore


drop if Name == "Total Value Added"

drop Name IOCode


save Use_Table_clean.dta, replace
export excel UseTable.xlsx, replace
	
cd ..

/**************************************************************************
2. Loads BEA-BLS Production Account Dataset and Merges it with BEA I-O Data
***************************************************************************/

clear all
cd Raw_Data
cd BEA_BLS_Production_Account

import excel BEA-BLS-industry-level-production-account-1987-2019, cellrange(A2:AH65) sheet(Integrated MFP Index) firstrow


** The BEA IO data is more disaggregated than the BEA-BLS Production Account
** Use the aggregated productivity data for the more disaggregated industry

/*
Disaggregate "Retail trade" to "Motor vehicle and parts dealers", "Food and beverage stores",
"General merchandise stores", and "Other retail"
*/

expand 2 if IndustryDescription=="Retail trade", generate(Retail_motor)
replace IndustryDescription="Motor vehicle and parts dealers" if Retail_motor==1
drop Retail_motor

expand 2 if IndustryDescription=="Retail trade", generate(Retail_food)
replace IndustryDescription="Food and beverage stores" if Retail_food==1
drop Retail_food

expand 2 if IndustryDescription=="Retail trade", generate(Retail_general)
replace IndustryDescription="General merchandise stores" if Retail_general==1
drop Retail_general

replace IndustryDescription="Other retail" if IndustryDescription=="Retail trade"

/*
Disaggregate "Real estate" to "Housing" and "Other real estate"
*/

expand 2 if IndustryDescription=="Real estate", generate(RE_housing)
replace IndustryDescription="Housing" if RE_housing==1
drop RE_housing
replace IndustryDescription="Other real estate" if IndustryDescription=="Real estate"

/*
Disaggregate "Hospitals and nursing and residential care facilities" to "Hospitals" and "Nursing and residential care facilities"
*/
expand 2 if IndustryDescription=="Hospitals and nursing and residential care facilities", generate(Hospitals)
replace IndustryDescription="Hospitals" if Hospitals==1
drop Hospitals
replace IndustryDescription="Nursing and residential care facilities" if IndustryDescription=="Hospitals and nursing and residential care facilities"

* Drop government industries
drop if IndustryDescription=="Federal" 
drop if IndustryDescription=="State and local" 


* Merge data with BEA IO data 
cd ../../Merged_Data
save productivity_temp.dta, replace 
clear
import delimited IndustryList.csv
rename v1 IOCode
rename v2 IndustryDescription
save IndustryList.dta , replace
outsheet IndustryDescription using "IndustryList.csv" , comma replace  
generate IndustryOrder=_n  
merge 1:1 IndustryDescription using productivity_temp.dta
sort IndustryOrder
drop IndustryOrder
drop _merge 
erase productivity_temp.dta

/* 
Output is the matrix of sectoral Integrated MFP index 1987-2019
The index is normalized to 100 for 2012
Industries appear in the same order as file "IndustryList.csv"
*/
save TFP.dta, replace
drop IOCode IndustryDescription
export excel TFP.xlsx, replace
cd ..

/**************************************************************************
3. Merge with Price Frequency Data from Pastern, Schoenle, and Weber (2020)
***************************************************************************/

/* 
Note: The price frequency dataset of Pasten et al. (2020) is based on 3-digit NAICS codes.
Using the concordance between NAICS codes and IOCodes, we manualy transformed the data
to IOCodes. The result is saved in file "FrequencyIOCode.xlsx". The original data can
be found in "FrequencyNAICS-3d.csv".
*/

* Load the price change frequencies
clear all
cd Raw_Data/Price_Frequency_Data
import excel FrequencyIOCode.xlsx, firstrow
save FrequencyIOCode.dta, replace


* Match industry names
clear all
cd ../../Merged_Data
use IndustryList.dta
generate IndustryOrder=_n  
cd ../Raw_Data/Price_Frequency_Data
merge 1:1 IOCode using FrequencyIOCode.dta
sort IndustryOrder

* Replace missing values with average frequency of other industries
egen mymean = mean(Frequency)
replace Frequency = mymean if missing(Frequency)
drop IndustryOrder mymean _merge

* Save final output
cd ../../Merged_Data
save Frequency.dta, replace
drop IOCode IndustryDescription
export excel Frequency.xlsx, replace


