
%% Main Code for La'O and Tahbaz-Salehi, "Optimal Monetary Policy in Production Networks," Econometrica (2022)
%  This code generates Table 1 and Figure 3 in the paper
%  It has to be run after running "Clean_and_Merge_Data.do", which cleans
%  and merges various data sources

%  To generate Table C.1 in the Online Appendix run ExtendedCode.m

clear;
clc;
close all


addpath('functions');
cd Merged_Data


global A n theta alpha Sigmaz L Domar beta

%% Reading the Make and the Use Table from BEA data and constructing the IO matrix

% Loading the Make and Use Tables
UseTable = xlsread('UseTable.xlsx');
MakeTable = xlsread('MakeTable.xlsx');

% Calibrating the value-added to compensation for employees
Compensation = xlsread('CompensationEmployees');
ValueAdded = Compensation;

% Loading FinalUse
FinalUse = xlsread('FinalUse');

% Constructing the industry by industry direct requirements table
IndustryOutput = sum(UseTable,1) + ValueAdded;
CI_DirectRequirements = UseTable  * inv(diag(IndustryOutput));
ShareTable = MakeTable * inv(diag(sum(MakeTable,1)));

II_DirectRequirements = ShareTable * CI_DirectRequirements;

% Input-output matrix
A = transpose(II_DirectRequirements);
n = length(A);

% Leontief inverse
L = inv(eye(n) - A);

% Labor share
alpha = ones(n,1) - sum(A,2);

% Domar weights
GDP = sum(ValueAdded);
Domar = transpose(IndustryOutput/GDP);

% Consumption shares
beta_temp = ShareTable * FinalUse/GDP;
beta_temp = max(beta_temp,0);
beta = beta_temp./(sum(beta_temp));

% Clean up variables no longer necessary
clear UseTable MakeTable beta_temp CI_DirectRequirements ShareTable II_DirectRequirements 

% Add a pseudo-industry that transforms labor into labor services
n = n+1;
A = [zeros(1,n) ; alpha, A];
alpha = [1; zeros(n-1,1)];
L = inv(eye(n) - A);
beta = [0;beta];
Domar = L' *beta ;

%% Sectoral TFP shocks using Integrated MFP from BEA/BLS Integrated Industry-level Production Account

MFP = xlsread('TFP.xlsx');
log_MFP = log(MFP);

% interpolate to transform annual data to quarterly data
t = [1:size(log_MFP,2)];
log_MFP_quarterly = zeros(n-1,4*(size(log_MFP,2)-1)+1);
for i=1:n-1
    log_MFP_quarterly(i,:) = interp1(t, log_MFP(i,:), [1:0.25: length(t)]);
end

% detrending log productivity sequences
log_MFP_linear_detrended = detrend(log_MFP_quarterly');

% variance-covariance matrix of shocks
Sigmaz = cov(log_MFP_linear_detrended);
Sigmaz = [zeros(1,n); [zeros(n-1,1), Sigmaz]];

%% Drawing log-productivity shocks with covariance matrix Sigmaz

% number of iterations
iteration = 10000;

% initialize the random number generator to make the results reproducable
rng('default');
rng(1);

% drawing productivity shocks with variance-covariance matrix Sigmaz
logz = mvnrnd(zeros(n,1),Sigmaz,iteration);
logz = logz';

%% Price flexibilities/stickiness (excluding wage stickiness)

% Load the FPA data from Pasten et al. (2020)
FPA = xlsread('Frequency.xlsx');
disp('mean of price change frequency');
disp(mean(FPA));
disp('median of price change frequency');
disp(median(FPA));

% expected price durations implied by FPA (in months)
exp_price_duration = 1./FPA;
disp('mean expected price duration')
disp(mean(exp_price_duration));
disp('median expected price duration');
disp(median(exp_price_duration));

% transforming the data from monthly to quarterly and calculating the
% degree of price flexibility of each industry
phi_ex_labor = 1 - exp(-3*FPA);   


%% Loading the list of industries

opts.DataLines = [2 inf];
Industry_list = readtable('IndustryList.csv');
Industry_list = [{'labor'} ; Industry_list];
Industry_list = categorical(Industry_list.IndustryDescription);
sectorlabels = reordercats(Industry_list,string(Industry_list));

%% Baseline calibration: Generating Table 1 and Figure 3

% preference parameters
eta = 2;        
gamma = .1;

% elasticity of substitution
theta = 6;

% degree of wage flexibility
phi_labor = 0.3; 
phi = [phi_labor; phi_ex_labor];

%% Calculating the weights corresponding to various policies
cd ..

% Optimal Policy
psi_optimal = optimalpolicy(eta, gamma, phi);

% Output Gap Stabilization Policy
psi_OG_stabilization = 1./phi .* (1-phi) .* Domar;
psi_OG_stabilization = psi_OG_stabilization/(sum(psi_OG_stabilization));

% Consumption price index targeting
psi_CPI = beta;

% Sales-weighted policy
psi_Domar = Domar;
psi_Domar(1) = 0;  % excluding wage
psi_Domar = psi_Domar/(sum(psi_Domar));

% No-Network Policy
psi_NoNetwork = (1./phi-1).* beta .*(beta'*phi -phi) + (1-phi).*beta*theta + (1./phi-1).* beta * (1-beta'*phi)/(gamma+1/eta);
psi_NoNetwork = psi_NoNetwork/(sum(psi_NoNetwork));

%% Calculating the welfare loss for each policy (reported in Table 1)

disp('Generating Table 1 and Figure 3 ...');

OptimalPolicy_WeflareLoss_Exact = expected_welfareloss_nonlinear(psi_optimal, logz, eta, gamma, phi);
OG_WeflareLoss_Exact = expected_welfareloss_nonlinear(psi_OG_stabilization, logz, eta, gamma, phi);
CPI_WeflareLoss_Exact = expected_welfareloss_nonlinear(psi_CPI, logz, eta, gamma, phi);
Domar_WeflareLoss_Exact = expected_welfareloss_nonlinear(psi_Domar, logz, eta, gamma, phi);
NoNetwork_WeflareLoss_Exact = expected_welfareloss_nonlinear(psi_NoNetwork, logz, eta, gamma, phi);

% NGDPT_WelfareLossExact = expected_welfareloss_nonlinear_NGDPT(logz, eta, gamma, phi);

%% Calculating the quadratic approximation to welfare loss for each policy (reported in Table 1)

[OptDelta , OptOG , OptAcross, OptWithin]          =  welfarelossQuadratic(psi_optimal, eta, gamma, phi);
[OGDelta , OGOG , OGAcross, OGWithin]              =  welfarelossQuadratic(psi_OG_stabilization, eta, gamma, phi);
[CPIDelta , CPIOG , CPIAcross, CPIWithin]           =  welfarelossQuadratic(psi_CPI, eta, gamma, phi);
[DomarDelta , DomarOG , DomarAcross, DomarWithin]   =  welfarelossQuadratic(psi_Domar, eta, gamma, phi);
[NoNetworkDelta , NoNetworkOG , NoNetworkAcross, NoNetworkWithin]   =  welfarelossQuadratic(psi_NoNetwork, eta, gamma, phi);

%% Cosine similarities (reported in Table 1)

cosine_opt_opt = cossim(psi_optimal,psi_optimal);
cosine_opt_OG = cossim(psi_optimal,psi_OG_stabilization);
cosine_opt_CPI = cossim(psi_optimal,psi_CPI);
cosine_opt_Domar = cossim(psi_optimal,psi_Domar);
cosine_opt_NoNetwork = cossim(psi_optimal,psi_NoNetwork);


%% Generate Table 1

% Welfare loss (fully nonlinear model)
Table1_WelfareLoss = [OptimalPolicy_WeflareLoss_Exact, OG_WeflareLoss_Exact, CPI_WeflareLoss_Exact, Domar_WeflareLoss_Exact, NoNetwork_WeflareLoss_Exact];
disp('Welfare Loss (full nonlinear model)');
disp(Table1_WelfareLoss);

% Quadratic approximation to welfare loss and various components
Table1_QuadraticLoss = [[OptDelta , OptWithin, OptAcross, OptOG]', [OGDelta , OGWithin, OGAcross, OGOG]', [CPIDelta , CPIWithin, CPIAcross, CPIOG]',[DomarDelta , DomarWithin, DomarAcross, DomarOG]', [NoNetworkDelta , NoNetworkWithin, NoNetworkAcross, NoNetworkOG]'];
disp('Welfare Loss (Quadratic Approximation) and misallocation components');
disp(Table1_QuadraticLoss);

% cosine similarity between different policies
Table1_Cossim = [cosine_opt_opt ; cosine_opt_OG ; cosine_opt_CPI ; cosine_opt_Domar ; cosine_opt_NoNetwork]';
disp('cosine similarity to optimal policy');
disp(Table1_Cossim);

%% Displaying other numbers reported in Section 5

% cosine similarity between various components of optimal policy
rho = inv(eye(n) - A*diag(phi)) * alpha;    % degree of upstream flexibility
rho_0 = beta' * diag(phi) * rho;
psi_og = 1./phi .* (1-phi) .* Domar * (1-rho_0)/(gamma+ 1/eta);
psi_within = theta .*  (1 - phi) .* Domar .* rho;
psi_across = (inv(diag(phi)) - eye(n)) * (rho_0 * Domar - diag(rho)*Domar + L' * (eye(n) - diag(phi))*diag(Domar) * rho );
cosine_og_within = cossim(psi_og,psi_within);
disp('cosine similarity between OG and within components');
disp(cosine_og_within);

% contributions of various policy components to optimal policy weights
psi_within_fraction = sum(psi_within)/sum(psi_within+psi_across+psi_og);
psi_across_fraction = sum(psi_across)/sum(psi_within+psi_across+psi_og);
disp('Contributions of within component to optimal policy weights');
disp(psi_within_fraction);
disp('Contributions of across component to optimal policy weights');
disp(psi_across_fraction);




%% Generate Figure 3

% Aggregating the weights 
psi_optimal_sectoral = sector_aggregate(psi_optimal);
psi_OG_sectoral = sector_aggregate(psi_OG_stabilization);

% Sectoral list (20 sectors, plus labor )
aggregated_sectoral_list = [{'Labor'};{'Agriculture, forestry, fishing, and hunting'} ; {'Mining'};  {'Utilities'}; {'Construction'}; ...
    {'Durable goods'} ; {'Nondurable goods'}; {'Wholesale trade'}; {'Retail trade'}; {'Transportation & warehousing'}; ...
    {'Information'}; {'Finance and insurance'}; {'Real estate, rental, and leasing'}; ... 
    {'Professional and technical services'}; {'Management of companies & enterprises'}; {'Administrative & waste services'}; {'Educational services'}; {'health care, and social assistance'};...
    {'Arts, entertainment, and recreation'}; {'Accommodation, and food services'}; {'Other services, except government'};];
aggregated_sectoral_list = categorical(aggregated_sectoral_list);
aggregated_sectoral_list = reordercats(aggregated_sectoral_list,string(aggregated_sectoral_list));


% Figure 3
bar(aggregated_sectoral_list, [psi_optimal_sectoral, psi_OG_sectoral]);
legend('Optimal Policy','OG Stabilization')


