%% This function determines the exact welfare under a given policy and realization of shocks
%  inputs are the realized productivity shocks and the policy weights 

function [welfare_loss_SS , exitflag] = expost_welfareloss_nonlinear(logz,psi,eta, gamma, phi)

global A n theta alpha Sigmaz L Domar beta 


%% Solving for equilibrium nominal prices and wage given policy and shocks

% steady-state nominal price as initial condition 
x0 = zeros(n,1);

% solve for equilibrium nominal prices for a given vector of shocks and policy weights
T = @(x)SolveEqNominalPrice(x,logz,psi,phi);
options = optimoptions('fsolve','Display','off');
[logp, ~, exitflag]= fsolve(T,x0,options);
if imag(logp)~=0 
    exitflag = -10;
end

% log nominal wage
logw = 1/(1-theta) * psi'* L * (log(1 - (1-phi).* exp((theta-1)*logp))-log(phi) + (1-theta)*logz);

% log nominal price of consumption good bundle
logp_cons_sticky = beta'*logp;
logp_cons_flexible = logw - beta'*L*logz;

%% vector of sectoral wedges 

log_mc = -logz + alpha*logw + A*logp;
epsilon = exp(log_mc + (theta-1)*logp) .* (phi.* exp(-theta*log_mc) + ones(n,1) - phi);

%% Sticky-price Domar weights

lambda_sticky = inv(eye(n) - A'*diag(epsilon))*beta;

%% Welfare loss given realized shocks and policy

if gamma ~=1 
    welfare_term1 = exp(((gamma-1)*(1+1/eta)/(gamma+1/eta)) * (logp_cons_sticky - logp_cons_flexible));
    welfare_term2 = (1 - lambda_sticky'*(ones(n,1) - epsilon))^((gamma-1)/(1+eta*gamma));
    welfare_term3 = 1 + (1-gamma)/(gamma+1/eta) * lambda_sticky' * (ones(n,1) - epsilon);

    % welfare under flexible prices
    welfare_flexible = (gamma+1/eta)/((1-gamma)*(1+1/eta)) * exp(((1-gamma)*(1+1/eta)/(gamma+1/eta)) * beta'*L*logz);

    % welfare under sticky prices
    welfare_loss = welfare_flexible * (1 - welfare_term1 * welfare_term2 * welfare_term3);
    
    % create an error flag if the solution is imaginary
    if imag(welfare_loss)~=0 
        exitflag = -10;
    end
    
    % welfare loss as a fraction of a steady-state GDP
    welfare_loss_SS = 1-(1 - (1-gamma)* welfare_loss)^(1/(1-gamma));
    
    % create an error flag if the solution is imaginary
    if imag(welfare_loss_SS)~=0 
        exitflag = -10;
    end
end

if gamma == 1 
    welfare_term1 = -(logp_cons_sticky - logp_cons_flexible);
    welfare_term2 = -1/(1+eta) * log(1 - lambda_sticky'*(ones(n,1) - epsilon));
    welfare_term3 = eta/(1+eta) * lambda_sticky'*(ones(n,1) - epsilon); 
    
    % welfare loss due to sticky prices
    welfare_loss = -(welfare_term1 + welfare_term2 + welfare_term3);
    
    % create an error flag if the solution is imaginary
    if imag(welfare_loss)~=0 
        exitflag = -10;
    end
    
    % welfare loss as a fraction of a steady-state GDP
    welfare_loss_SS = 1-exp(-welfare_loss);
   
    
end


end
